#ifndef __FTP_H__
#define __FTP_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#define FTP_WANT_NONE			0x000
#define FTP_WANT_POLL_IN		0x001
#define FTP_WANT_POLL_OUT		0x002
#define FTP_WANT_POLL_DATA_IN	0x004
#define FTP_WANT_POLL_DATA_OUT	0x008

enum {
	FTP_ERROR = -1,
	FTP_POLL = 0,
	FTP_SUCCESS = 1
};

enum {
	FTP_DATA_NONE = 0,
	FTP_DATA_READ,
	FTP_DATA_WRITE
};

enum {
	FTP_STATE_CLOSED = 0,
	FTP_STATE_TRYING_CONNECT,
	FTP_STATE_CONNECTED,
	FTP_STATE_LOGIN_USER,
	FTP_STATE_LOGIN_PASS,
	FTP_STATE_LOGGED,
	FTP_STATE_CMD,
	FTP_STATE_PASV,
	FTP_STATE_CLOSING
};

typedef struct _FTPSession FTPSession;

FTPSession *ftp_session_create (void);

typedef void (*FTPLineFunc) (FTPSession *session, const char *line, void *data);
int ftp_session_connect (FTPSession *session, struct sockaddr *server, socklen_t addr_len);
int ftp_session_login (FTPSession *session, const char *user, const char *passwd);
int ftp_session_send_command (FTPSession *session, char *command, int data_dir);
int ftp_session_send_close (FTPSession *session);
int ftp_session_request_pasv (FTPSession *session, FILE *stream);
void ftp_session_close_and_clean (FTPSession *session);

int ftp_session_async_continue (FTPSession *session, int what);
int ftp_session_get_state (FTPSession *session);
int ftp_session_get_poll (FTPSession *session);
int ftp_session_get_control_socket (FTPSession *session);
int ftp_session_get_data_socket (FTPSession *session);
int ftp_session_get_last_code (FTPSession *session);
char * ftp_session_get_last_message (FTPSession *session);
const char *ftp_session_get_user (FTPSession *session);

void ftp_session_free (FTPSession *session);

void ftp_session_set_multiline_func (FTPSession *session, FTPLineFunc func, void *data);

#endif /* __FTP_H__ */
